@@ -1,8 +1,9 @@
+#[allow(unused_imports)]
 #[macro_use]
 extern crate assert_float_eq;
 
 #[readonly::make]
-#[derive(Debug, PartialEq)]
+#[derive(Clone)]
 pub struct DelayTimes {
     pub v_whole: f64,
     pub v_half: f64,
@@ -14,23 +15,20 @@ pub struct DelayTimes {
     pub v_128th: f64,
 }
 
+// Using a state pattern design with `DelayTimes` and `DelayTimesModifier` to make sure the user
+// can't repeatedly call certain functions, like `tripler()`
+
 impl DelayTimes {
-    pub fn in_ms(beats_per_minute: f64) -> Self {
+    pub fn in_ms(beats_per_minute: f64) -> DelayTimesModifier {
         let ms: f64 = 60_000.0 / beats_per_minute;
-        DelayTimes::get_instance(ms)
+        let delay_times = DelayTimes::get_instance(ms);
+        DelayTimesModifier::new(delay_times)
     }
 
-    pub fn in_hz(beats_per_minute: f64) -> Self {
+    pub fn in_hz(beats_per_minute: f64) -> DelayTimesModifier {
         let hz: f64 = beats_per_minute / 60.0;
-        DelayTimes::get_instance(hz)
-    }
-
-    pub fn dotted(&self) -> Self {
-        self.multiply_all_values_by(1.5)
-    }
-
-    pub fn triplet(&self) -> Self {
-        self.multiply_all_values_by(2.0 / 3.0)
+        let delay_times = DelayTimes::get_instance(hz);
+        DelayTimesModifier::new(delay_times)
     }
 
     fn get_instance(base_value: f64) -> Self {
@@ -45,17 +43,39 @@ impl DelayTimes {
             v_128th: base_value / 32.0,
         }
     }
+}
 
-    fn multiply_all_values_by(&self, multiplier: f64) -> Self {
-        Self {
-            v_whole: self.v_whole * multiplier,
-            v_half: self.v_half * multiplier,
-            v_quarter: self.v_quarter * multiplier,
-            v_8th: self.v_8th * multiplier,
-            v_16th: self.v_16th * multiplier,
-            v_32nd: self.v_32nd * multiplier,
-            v_64th: self.v_64th * multiplier,
-            v_128th: self.v_128th * multiplier,
+pub struct DelayTimesModifier {
+    delay_times: DelayTimes,
+}
+
+impl DelayTimesModifier {
+    fn new(delay_times: DelayTimes) -> Self {
+        Self { delay_times }
+    }
+
+    pub fn normal(&self) -> DelayTimes {
+        self.delay_times.clone()
+    }
+
+    pub fn dotted(&self) -> DelayTimes {
+        DelayTimesModifier::multiply_all_values_by(self, 1.5)
+    }
+
+    pub fn triplet(&self) -> DelayTimes {
+        DelayTimesModifier::multiply_all_values_by(self, 2.0 / 3.0)
+    }
+
+    fn multiply_all_values_by(&self, multiplier: f64) -> DelayTimes {
+        DelayTimes {
+            v_whole: self.delay_times.v_whole * multiplier,
+            v_half: self.delay_times.v_half * multiplier,
+            v_quarter: self.delay_times.v_quarter * multiplier,
+            v_8th: self.delay_times.v_8th * multiplier,
+            v_16th: self.delay_times.v_16th * multiplier,
+            v_32nd: self.delay_times.v_32nd * multiplier,
+            v_64th: self.delay_times.v_64th * multiplier,
+            v_128th: self.delay_times.v_128th * multiplier,
         }
     }
 }
@@ -96,7 +116,7 @@ mod tests {
                 v_128th: 15.625,
             };
 
-            let actual_delay_times = DelayTimes::in_ms(120.0);
+            let actual_delay_times = DelayTimes::in_ms(120.0).normal();
 
             assert_delay_times_instances_are_equal(&expected_delay_times, &actual_delay_times)
         }
@@ -155,7 +175,7 @@ mod tests {
                 v_128th: 0.0625,
             };
 
-            let actual_delay_times = DelayTimes::in_hz(120.0);
+            let actual_delay_times = DelayTimes::in_hz(120.0).normal();
 
             assert_delay_times_instances_are_equal(&expected_delay_times, &actual_delay_times)
         }
@@ -196,4 +216,30 @@ mod tests {
             assert_delay_times_instances_are_equal(&expected_delay_times, &actual_delay_times)
         }
     }
+
+    // Here are a couple of weird tests that just ensures we don't break the interface
+    // They can't fail, but at least the code won't compile if something about the inferface changes
+    mod interface_tests {
+        #[test]
+        fn test_single_shot() {
+            use super::DelayTimes;
+
+            let _delay_times = DelayTimes::in_ms(120.0).normal();
+
+            assert!(true)
+        }
+
+        #[test]
+        fn test_reusability() {
+            use super::DelayTimes;
+
+            let delay_times_modifier = DelayTimes::in_ms(120.0);
+
+            let _delay_times_normal = delay_times_modifier.normal();
+            let _delay_times_dotted = delay_times_modifier.dotted();
+            let _delay_times_triplet = delay_times_modifier.triplet();
+
+            assert!(true)
+        }
+    }
 }
